function [pos neg test] = UCI_data(name, testVideo)
% this function is very dataset specific, you need to modify the code if
% you want to apply the pose algorithm on some other dataset

% it converts the various data format of different dataset into unique
% format for pose detection 
% the unique format for pose detection contains below data structure
%   pos:
%     pos(i).im: filename for the image containing i-th human 
%     pos(i).point: pose keypoints for the i-th human
%   neg:
%     neg(i).im: filename for the image contraining no human
%   test:
%     test(i).im: filename for i-th testing image
% This function also prepares flipped images and slightly rotated images for training.

globals;
  
cls = [name '_data'];
try
	load([cachedir cls]);
catch
    
    folder{1} = 'DATA_DEPTH_V1/';   % testVideo = 1;
    folder{2} = 'DATA_DEPTH_V2/';   % testVideo = 2;
    folder{3} = 'DATA_DEPTH_V3/';   % testVideo = 3;
    folder{4} = 'DATA_DEPTH_V4/';   % testVideo = 4;
    folder{5} = 'DATA_DEPTH_V5/';   % testVideo = 5;
    folder{6} = 'DATA_DEPTH_V6/';   % testVideo = 6;

    data_path = './uci_data/';
    
    numIM = 400;    % We use 400 images 
    %numIMNeg = 5;   % 5 negatives for each image
    
    % -------------------
    % grab positive annotation and image information
    
    pos = [];
    numpos = 0;
    
    %negims = './uci_data/NEGATIVES/V%d/n%.5d.jpg';
    neg = [];
    numneg = 0;
    
    %%%%rand('state',1),ind=randperm(length(images))
    
    for trainVideo = setdiff(1:length(folder), testVideo)
        
        path = [data_path folder{trainVideo}];
        images = dir([path '/*jpg']);

        ind = randperm(length(images)); % We choose the 'numIM' images randomly
  
        for j = 1:numIM
            if j <= length(images)
                numpos = numpos + 1;
                fprintf('TRAIN: %d - %s\n', numpos, [path, images(ind(j)).name]);
                p = load([path '' images(ind(j)).name(1:end-4) '_labels.mat']);
                labels = p.labels;
                pos(numpos).im = [path, images(ind(j)).name];%[sprintf(posims,numpos) '.jpg'];
                pos(numpos).point = labels;
                
                numneg = numneg + 1;
                fprintf('NEG: %d - %s\n', numneg, [path 'NEG/' images(ind(j)).name]);
                neg(numneg).im = [path 'NEG/' images(ind(j)).name];
                % grab negative image information
                %for i = 1:numIMNeg
                %    numneg = numneg + 1;
                %   fr = i + (5 * (ind(j) - 1)); % 5 negatives for each image
                %    neg(numneg).im = sprintf(negims,trainVideo,fr);
                %end
                
            end
        end
    end
        
 
  % -------------------
  % rotate positive images by a small amount of degree
  degree = [-15 -7.5 7.5 15];
  posims_rotate = [cachedir 'imrotate/im%.4d_%d.jpg'];
  for n = 1:length(pos)
    im = imread(pos(n).im);
    for i = 1:length(degree)
      fprintf('ROTATING - (%d/%d)(%d) - %s\n', n, length(pos), i, pos(n).im);
      imwrite(imrotate(im,degree(i)),sprintf(posims_rotate,n,i));
    end
  end

  for n = 1:length(pos)
    im = imread(pos(n).im);
    for i = 1:length(degree)
      numpos = numpos + 1;
      pos(numpos).im = sprintf(posims_rotate,n,i);
      pos(numpos).point = map_rotate_points(pos(n).point,im,degree(i),'ori2new');
    end
  end       
  
  
  %% NEGATIVES
  %{
  neg = [];
    numneg = 0;
    for trainVideo = setdiff(1:length(folder), testVideo)
        path = [data_path folder{trainVideo}];
        images = dir([path '/*jpg']);

        ind = randperm(length(images)); % We choose the 'numIM' images randomly
        for j = 1:numIM%length(images)
            if j <= length(images)
                numneg = numneg + 1;
                fprintf('NEGATIVE: %d - %s\n', numpos, [path, images(ind(j)).name]);
                neg(numneg).im = [path, images(ind(j)).name];
            end
        end
    end
  %}
  %{
  %% NEG
    % -------------------
    % grab negative image information
    %negims = 'uci_data/NEGATIVES';
    %negativesDir = dir([negims '/*jpg']);
    ind = randperm(length(negativesDir));
    
    
    neg(numneg).im = sprintf(negims,trainVideo,fr);
    neg = [];
    numneg = 0;
    
    %for i = 1:
    %    neg(i).im = negativesDir(ind(i)).name;
    %end
    
    for trainVideo = setdiff(1:length(folder), testVideo)
        for fr = 1:numIM*5 % We have generated 5 negatives for image
            numneg = numneg + 1;
            neg(numneg).im = sprintf(negims,trainVideo,fr);
        end
        
    end
%}
  
  %{
  
  % INRIA
    trainfrs_neg = 615:1832; % training frames for negative 
    %%
    % -------------------
    % grab negative image information
    negims = 'uci_data/INRIA_NEGATIVES/%.5d.jpg';
    neg = [];
    numneg = 0;
    for fr = trainfrs_neg
        numneg = numneg + 1;
        neg(numneg).im = sprintf(negims,fr);
    end
    %}
  
  
  
  
  
  
    % -------------------
    % grab testing image information 
    path = [data_path folder{testVideo}];
    images = dir([path '/*jpg']);
    
    test = [];
    numtest = 0;
  
    for j = 1:length(images)
        numtest = numtest + 1;
        
        fprintf('TEST: %d - %s\n', numtest, [path, images(j).name]);
        
        p = load([path '' images(j).name(1:end-4) '_labels.mat']);
        labels = p.labels;
            
        test(numtest).im = [path images(j).name];
        test(numtest).point = labels;
    end
    
    save([cachedir cls],'pos','neg','test');
end
