function UCI_demo(testVideo)
    %clc; close all; clear all;
    %tStart = tic;
    globals;
    name = 'UCI';
    % --------------------
    % specify model parameters
    % number of mixtures for 12 parts
    K = [6 6 6 6 6 6 6 6 6 6 6 6];
    % Tree structure for 12 parts: pa(i) is the parent of part i
    % This structure is implicity assumed during data preparation
    % (PARSE_data.m) and evaluation (PARSE_eval_pcp)
    pa = [0 1 2 3 4 5 6 2 8 9 10 11];
    % Spatial resolution of HOG cell, interms of pixel width and hieght
    sbin = 8;
    % --------------------
    % Prepare training and testing images and part bounding boxes
    [pos neg test] = UCI_data(name, testVideo);
    pos = point2box(pos,pa);

    % --------------------
    % training
    model = trainmodel(name,pos,neg,K,pa,sbin);
    % --------------------
    % testing phase 1
    % human detection + pose estimation
    suffix = num2str(K')';
    model.thresh = min(model.thresh,-2);
    boxes = testmodel(name,model,test,suffix);
    det = UCI_transback(boxes);
    % --------------------
    % evaluation 1: average precision of keypoints
    % You will need to write your own APK evaluation code for your data structure
    apk = eval_apk(det, test, 0.2);

    meanapk = mean(apk);
    fprintf('mean APK = %.1f\n',meanapk*100);
    fprintf('APK       '); fprintf('& %.1f ',apk*100); fprintf('\n');
    
    % --------------------
    % visualization
    re = sprintf('./res/V%d/', testVideo);
    mkdir(re);
    mkdir([re 'boxes/']);
    mkdir([re 'skeletons/']);
    
    figure(1);
    visualizemodel(model);
    print([re '_model.jpg'],'-djpeg');
    figure(2);
    visualizeskeleton(model);
    print([re '_skeleton.jpg'],'-djpeg');
    
    close all;
    res = [re '%s/%.4d.jpg'];
    
    for demoimid=1:10%size(test,2)

        fprintf('(%d / %d)\n', demoimid, size(test,2));
        
        im = imread(test(demoimid).im);
        colorset = {'g','g','y','r','r','r','c','y','b','b','b','c'};
        box = boxes{demoimid};
        % show all detections
        figure(3);
        showboxes(im,box,colorset);
        print(sprintf(res,'boxes',demoimid),'-djpeg');
        showskeletons(im,box,colorset,model.pa);
        print(sprintf(res,'skeletons',demoimid),'-djpeg');
        
        close;
        pause(0.05);

    end

end
    