%% Import libraries
addpath('./lib/classifiers/libsvm_primal/');
addpath('./lib/descriptors/');

tStart = tic;

%% Parameters (to be changed for different tests)
testing = 1;  str_test = 'HEAD'; p1 = 1; p2 = 2;
%testing = 2;  str_test = 'L_HAND'; p1 = 12; p2 = 12;
%testing = 3;  str_test = 'R_HAND'; p1 = 7; p2 = 7;

%descriptor = 1; str_desc = 'HOG';
%descriptor = 2; str_desc = 'DEPTH';
descriptor = 3; str_desc = 'HOG_DEPTH';

%%
fprintf(['Testing ' str_test ' with ' str_desc '\n']);

%Image folders
folder{1} = 'DATA_DEPTH_V1/';   % testVideo = 1;
folder{2} = 'DATA_DEPTH_V2/';   % testVideo = 2;
folder{3} = 'DATA_DEPTH_V3/';   % testVideo = 3;
folder{4} = 'DATA_DEPTH_V4/';   % testVideo = 4;
folder{5} = 'DATA_DEPTH_V5/';   % testVideo = 5;
folder{6} = 'DATA_DEPTH_V6/';   % testVideo = 6;

data_path = './uci_data/';

path_to_save = ['./Results/', str_test, '/', str_desc, '/'];
if ~exist(path_to_save,'dir')
    mkdir(path_to_save);
end

global X    % For SVM

%for testVideo = 1:length(folder)
    testVideo = 2;
    fprintf('BOOTSTRAPING VIDEO %s %s\n', num2str(testVideo), str_test);
    path = [data_path folder{testVideo}];
        
    try
        load([path_to_save 'TV_' num2str(testVideo) '_' str_test '_IT_0']);
        fprintf('IT: 0 - AP: %.3f\n', ap);
        
    catch
        
        %% Construct Training data and Labels (without 'testVideo')
        [XPostraining XNegtraining] = train_trainingData(data_path, testVideo, [str_test '_'], p1, p2, descriptor, str_desc);
        
        Data = [XPostraining; XNegtraining];
        Labels = [ones(size(XPostraining,1),1); -ones(size(XNegtraining,1),1)];
    
        fprintf('Training SVM...\n');
        X = Data;
        [w b] = primal_svm(1,Labels, 1); % [SOL, B] = PRIMAL_SVM(LINEAR,Y,LAMBDA,OPT)

        filename = [path_to_save 'TV_' num2str(testVideo) '_IT_0.jpg'];
        
        [ap, boxes] = test(w,b,path,descriptor,p1,p2,filename);
        
        fprintf('IT: 0 - AP: %.3f\n', ap);
        
        save([path_to_save 'TV_' num2str(testVideo) '_' str_test '_IT_0'],'XNegtraining','XPostraining','ap','boxes');
    end
    
    %% 
    for iteration = 1:3
        
        try
            load([path_to_save 'TV_' num2str(testVideo) '_' str_test '_IT_' num2str(iteration)]);
            fprintf('IT: %d - AP: %.3f\n', iteration, ap);
        
        catch
            
            Data = [XPostraining; XNegtraining];
            Labels = [ones(size(XPostraining,1),1); -ones(size(XNegtraining,1),1)];

            X = Data;
            [w b] = primal_svm(1,Labels, 1); % [SOL, B] = PRIMAL_SVM(LINEAR,Y,LAMBDA,OPT)
            
            XNegtraining = train_addHardNegatives (data_path, folder, testVideo, w,b, XNegtraining,p1,p2, descriptor);
            
            fprintf('(IT: %d) - Training SVM Again...\n',iteration);

            Data = [XPostraining; XNegtraining];
            Labels = [ones(size(XPostraining,1),1); -ones(size(XNegtraining,1),1)];

            X = Data;
            [w b] = primal_svm(1,Labels, 1); % [SOL, B] = PRIMAL_SVM(LINEAR,Y,LAMBDA,OPT)

            filename = [path_to_save 'TV_' num2str(testVideo) '_IT_' num2str(iteration) '.jpg'];
            [ap, boxes] = test (w,b,path,descriptor,p1,p2,filename);
            
            fprintf('IT: %d - AP: %.3f\n', iteration, ap);
            
            save([path_to_save 'TV_' num2str(testVideo) '_' str_test '_IT_' num2str(iteration) ],'XNegtraining','XPostraining','ap','boxes');
        
        end

    end
%end
    
time = toc(tStart);
fprintf('Time elapsed: %.3f h\n', time/3600); 