%% Generate '.mat' with 'Descriptor' information
% descriptor = 1 HOG, 2 DEPTH, 3 HOG+DEPTH
function data = train_doTrain (data_path, val1, val2, filename, descriptor)

    %% External libraries
    addpath('./lib/descriptors/');

    %% Parameters
    box_widht   = 70./2;    % Box widht
    box_height  = 60.0/2;   % Box height

    % Feature vector length (to be change if descriptor is changed)
    sbin = 8;
    
    if descriptor == 1
        dim = (6*7*32);
    elseif descriptor == 2
        dim = (6*7*16);
    else
        dim = (6*7*32)+(6*7*16);
    end

    % Image folders
    folder{1} = 'DATA_DEPTH_V1/';
    folder{2} = 'DATA_DEPTH_V2/';
    folder{3} = 'DATA_DEPTH_V3/';
    folder{4} = 'DATA_DEPTH_V4/';
    folder{5} = 'DATA_DEPTH_V5/';
    folder{6} = 'DATA_DEPTH_V6/';

    %% Init variables
    data = {};

    %% For each video
    for i=1:size(folder,2)
        path = [data_path folder{i}];
        
        images = dir([path '/*jpg']);   % Load images for current folder
        points = dir([path '/*mat']);   % Load labels for current folder
        
        data{i}.xPos = zeros(length(images),dim);
        data{i}.xNeg = zeros(10*length(images),dim);

        for j=1:length(images),
            fprintf('%s\n',[path, images(j).name]);

            % Load an image and its labels
            imgT = imread([path, images(j).name]);
            imgT = imresize(imgT,0.5);    % Resize the img
            img(:,:,1) = imgT;
            img(:,:,2) = imgT;
            img(:,:,3) = imgT;
            label = load([path, points(j).name]);

            x = label.labels(:,1);
            y = label.labels(:,2);
            
            %% True box
            box = getBoxPartBody (x, y, box_widht, box_height, val1, val2);
            
            aux = img(box(2):box(4),box(1):box(3),:);
            
            if descriptor == 1
                feat = features(double(aux),sbin);
                desc = feat(:)';
            elseif descriptor == 2
                depth = depthDesc(double(aux),sbin);
                desc = depth(:)';
            else
                feat = features(double(aux),sbin);
                depth = depthDesc(double(aux),sbin);
                desc = [feat(:)' depth(:)'];
            end
            
            data{i}.xPos(j,:) = desc(:);

            % Negative
            for z=1:10,
                cordRand = train_boxOutOfRange(box(1),box(2),box_widht*2,box_height*2);

                aux = img(cordRand(2):cordRand(4),cordRand(1):cordRand(3),:);
                if descriptor == 1
                    feat = features(double(aux),sbin);
                    desc = feat(:)';
                elseif descriptor == 2
                    depth = depthDesc(double(aux),sbin);
                    desc = depth(:)';
                else
                    feat = features(double(aux),sbin);
                    depth = depthDesc(double(aux),sbin);
                    desc = [feat(:)' depth(:)'];
                end
                data{i}.xNeg((j-1)*10 + z,:) = desc(:);
            end

        end

    end

    % Save the features in a file
    fprintf('%Saving file [s]...\n',[data_path filename]);
    save([data_path, filename, '.mat'],'data');
end
